#!/bin/bash
#set -x

# Initialize variables
grub_shell=grub
grubdir=/boot/grub
efidir=/boot/efi/EFI
no_floppy="--no-floppy"

# lib or lib64 ?
if [ -f /opt/apprecovery/lib*/libext2fs.so ]; then
	libext2fs=$(dirname $(ls -1 /opt/apprecovery/lib*/libext2fs.so* | head -1))/libext2fs.so
fi

die_if_faulted() {
	echo "Boot fix repair failed on function $2"
	exit $1
}

continue_if_faulted() {
	echo "Boot fix repair failed on function $1"
}

install_boot_block() {
    # Before all invocations of the grub shell, call sync to make sure
    # the raw device is in sync with any bufferring in filesystems.
    sync

    $grub_shell --batch $no_floppy <<EOF 
root $1
setup --stage2=$grubdir/stage2 $2
quit
EOF
}

gen_net_devices()
{
    net_devices=($(ls /sys/class/net/))
    for device in ${net_devices[@]}; do
        case "$device" in
            lo) continue ;;
            *) touch /etc/sysconfig/network-scripts/ifcfg-$device
               ifcfg_file=/etc/sysconfig/network-scripts/ifcfg-$device
               uuid_net_dev=$(uuidgen $network_devices)
echo -n "TYPE=\"Ethernet\"
PROXY_METHOD=\"none\"
BROWSER_ONLY=\"no\"
BOOTPROTO=\"dhcp\"
DEFROUTE=\"yes\"
IPV4_FAILURE_FATAL=\"no\"
IPV6INIT=\"yes\"
IPV6_AUTOCONF=\"yes\"
IPV6_DEFROUTE=\"yes\"
IPV6_FAILURE_FATAL=\"no\"
IPV6_ADDR_GEN_MODE=\"stable-privacy\"
NAME=\"$device\"
UUID=\"$uuid_net_dev\"
DEVICE=\"$device\"
ONBOOT=\"yes\"
" > $ifcfg_file
            ;;
        esac
    done
}

install_grub_to_disks()
{
	for grub_disk in "${grub_disks[@]}"; do
		echo "Executing \"$grub_version-install $grub_disk\"."
		$grub_version-install $grub_disk || die_if_faulted 1 $1
	done
}

call_depmod_for_all_kernels()
{
	for d in `ls /lib/modules/` ; do
		depmod $d || continue_if_faulted "Failed to run 'depmod $d'"
	done
}

while getopts "h:p:g:f:b:n:d:v:r:eicmkloz" opt; do
	case $opt in
		# Disk nubmer for grub to install boot block (e.g. hd(x,1))
		h) disk_number=${OPTARG} ;;
		# Partition number for grub to install boot block (e.g. hd(0,x))
		p) part_number=${OPTARG} ;;
		# Disk name for grub to install (e.g. /dev/sd(x))
		g) OLD_IFS=$IFS
		   IFS=' '
		   grub_disks=($OPTARG) # use the split+glob operator
		   IFS=$OLD_IFS ;;
		# *.efi file that is specified in BIOS boot step to boot from (e.g. grubx64.efi)
		f) efi_file=${OPTARG} ;;
		# Caption of boot step in BIOS (just for pretty look of boot process)
		b) boot_caption=${OPTARG} ;;
		# Partition number where efi is located (e.g. /dev/sda(x))
		n) efi_part_number=${OPTARG} ;;
		# Directory inside efi boot directory that vary from os to os (e.g. centos/redhat/ubuntu/...)
		d) efi_os_dir=${OPTARG} ;;
		# Version on grub scripts to use. It don't depend on actual grub name, it depends on how it named in distrib (e.g. grub/grub2/...)
		v) grub_version=${OPTARG} ;;
		# Is restored system efi based
		e) is_efi=yes ;;
		# Should we install grub?
		i) install_grub=yes ;;
		# Should we remake configs?
		c) make_config=yes ;;
		# Should we call efibootmgr to add boot step in BIOS?
		m) call_efibootmgr=yes ;;
		# Options are: mkinitrd, update-initramfs, dracut.
		# - mkinitrd is needed only for SLES 11 non-EFI after installation of the boot block;
		# - update-initramfs is needed only for Ubuntu 16 with BTRFS;
		# - dracut is needed for CentOS when BMR-ed to XEN or KVM and not bad in other cases (hardware could be changed).
		r) rebuild_initramfs=${OPTARG} ;;
		# Should we update grub.cfg by the update-grub command?
		k) call_updategrub=yes ;;
		# Create a new network configuration it's need for rhel's family OS's to make OS bootable after BMR.
		l) create_lan_config=yes ;;
		# depmod should be called to add md raid drives to the kernel before update-initramfs.
		o) call_depmod=yes ;;
		# mdadm.conf should be refreshed before update of the grub configuration if "/" is located on the RAID.
		z) refresh_mdadm_config=yes ;;
	esac
done

if [ -z $is_efi ]; then
	if [ ! -z $install_grub ]; then
		if [ ! -z $grub_version ]; then
			install_grub_to_disks "grub-install (non-efi)"
		else
			echo "Installing boot block to the \"(hd$disk_number,$part_number)\"."
			install_boot_block "(hd$disk_number,$part_number)" "(hd$disk_number)" || die_if_faulted 2 "install_boot_block"
		fi
	fi

	if [ ! -z $make_config ]; then
		rm -rf /etc/grub.cfg /boot/$grub_version/grub.cfg
		echo "Executing \"$grub_version-mkconfig > /boot/$grub_version/grub.cfg\"."
		$grub_version-mkconfig > /boot/$grub_version/grub.cfg || die_if_faulted 3 "grub-mkconfig (BIOS)"
	fi
else
	if [ ! -z $install_grub ]; then
		install_grub_to_disks "grub-install (efi)"
	fi

	if [ ! -z $make_config ]; then
		rm -rf $efidir/$efi_os_dir/grub.cfg /etc/grub.cfg /boot/grub/grub.cfg /boot/grub2/grub.cfg
		echo "Executing \"$grub_version-mkconfig > $efidir/$efi_os_dir/grub.cfg\"."
		$grub_version-mkconfig > $efidir/$efi_os_dir/grub.cfg || die_if_faulted 3 "grub-mkconfig (efi)"
		if [ -d "/boot/grub" ]; then
			cp $efidir/$efi_os_dir/grub.cfg /boot/grub/grub.cfg
		fi
		if [ -d "/boot/grub2" ]; then
			cp $efidir/$efi_os_dir/grub.cfg /boot/grub2/grub.cfg
		fi
	fi
	
	if [ ! -z $call_efibootmgr ]; then
		echo "Executing \"efibootmgr -c -l \\efi\\$efi_os_dir\\$efi_file.efi -L \"$boot_caption\" -p $efi_part_number\"."
		efibootmgr -c -l \\efi\\$efi_os_dir\\$efi_file.efi -L "$boot_caption" -p $efi_part_number || die_if_faulted 4 "efibootmgr"
	fi
fi

if [ ! -z $refresh_mdadm_config ]; then
	echo "Refreshing /etc/mdadm/mdadm.conf."
	mkconf=/usr/share/mdadm/mkconf
	[ ! -f $mkconf ] && mkconf="mdadm --detail --scan"

	if [ -d "/etc/mdadm" ]; then
		mv -f /etc/mdadm/mdadm.conf /etc/mdadm/mdadm.conf.bmr
		$mkconf > /etc/mdadm/mdadm.conf || die_if_faulted 7 "call /usr/share/mdadm/mkconf > /etc/mdadm/mdadm.conf"
	fi
	
	if [ -f /etc/mdadm.conf ]; then
		mv -f /etc/mdadm.conf /etc/mdadm.conf.bmr 
		$mkconf > /etc/mdadm.conf || die_if_faulted 7 "call /usr/share/mdadm/mkconf > /etc/mdadm.conf"
	fi
fi

if [ ! -z $call_depmod ]; then
        echo "Calling depmod for all kernels."
	call_depmod_for_all_kernels
fi

if [ ! -z $rebuild_initramfs ]; then

	rebuild_cmd=""

	case "$rebuild_initramfs" in
		mkinitrd)         rebuild_cmd="mkinitrd" ;;
		update-initramfs) rebuild_cmd="update-initramfs -u -k all" ;;
		dracut)		  rebuild_cmd="dracut --force --regenerate-all" ;;
		*) 		  echo "ERROR: Parameter -r has incorrect value. It should be one of these: mkinitrd/update-initramfs/dracut."
				  return -1 ;;
	esac

	if [ ! -z "$libext2fs" ]; then
		libs="$(ls $libext2fs*)"
	fi

        for lib in $libs; do mv $lib $lib.bmr; done

	echo "Rebuilding kernels with $rebuild_initramfs."
	$rebuild_cmd || die_if_faulted 5 "call $rebuild_initramfs"

	for lib in $libs; do mv $lib.bmr $lib; done
fi

if [ ! -z $call_updategrub ]; then
        echo "Updating grub.cfg with update-grub."
        update-grub || die_if_faulted 6 "call update-grub"
fi

if [ -n "$create_lan_config" ]; then
	gen_net_devices
fi
