#/bin/bash
#set -x
#
# Global variables
#
typeset _root_device="none"
typeset _boot_device="none"
typeset _interactive_mode="off"
typeset _rootfs_path="/tmp/rootfs"

# Check of the return code of previous command
# input param: line to show it case of error
function success_or_die {
    [ $? -eq 0 ] || {
         echo "$1"
         exit 1
    } 
}

# Printing help text message
function help {
    echo "repair_system.sh -r <device name> [-b <device name> -i]"
    echo "Input Parameters: "
    echo "-r <device path> - Path to the device with root volume. Example -r /dev/sda2"
    echo " -b <device path> - Path to the device with boot volume. No need to pass this"
    echo "            parameter if boot dir is part of /root volume. Example -b /dev/sda1"
    echo "-i - run script in interactive mode. Ask user to enter devices for partitions if fstab"
}

# Doing all job of mappting stires for the device
# input params: device path & mount point(/dev/sda1 /tmp/tempfs for example)
function initialize_stores {
    # Commented creation bitmap and data store 
    # assuming that they are already created.
    # bsctl --create-data-store $1 &&
    # bsctl --create-bitmap-store $1 &&
    bsctl --check-data-store $1 2> /dev/null || {
        echo -e "Data store is not valid or absent. Recreating data store..."
        rm -f $2/.blksnap/data
        bsctl --create-data-store $1
    }
    bsctl --map-data-store $1

    bsctl --check-bitmap-store $1 2> /dev/null || {
        echo -e "Bitmap store is not valid or absent. Next snapshot will be basic"
        read -p "Do you want to proceed?[Y/n]:" answer
        if [ "$answer" == "n" ]; then
            echo "No valid bitmap! Aborted by user."
            return 1
        else
            rm -f $2/.blksnap/bitmap
            bsctl --create-bitmap-store $1
        fi
    }
    bsctl --map-bitmap-store $1
}

# Attaching device to blksnap
# input param: device path (/dev/sda1 for example)
function attach_device {
    bsctl -a $1  
}

# Detaching device from blksnap
# input param: device path (/dev/sda1 for example)
function detach_from_device {
    bsctl -d $1
}

# Cleaning function all temporary files, mounts etc.
# In case of non 0 input - undo all changes that were done.
# Input param: result code
function clean_and_exit {
    [ -e "/tmp/devices" ] && { 
        rm /tmp/devices
    }
    [ -e "$_rootfs_path/etc/fstab.backup" ] && {
        if [ $1 -ne 0 ]; then                                          # Restore fstab in case of error ($1 not equals 0)
            mv $_rootfs_path/etc/fstab.backup $_rootfs_path/etc/fstab
        else
            rm $_rootfs_path/etc/fstab.backup
        fi 
    }

    if [ $_boot_device != "none" ] && [ $(mount | grep -c ${_boot_device}) -ne 0 ]; then  # Check boot and root device. If it is still mounted - 
        detach_from_device ${_boot_device}                                                # unmount them and detach from blksnap
        success_or_die "Can not detach from ${_boot_device}"        

        umount ${_boot_device}
        success_or_die "Can not unmount boot device. Please try to do this by hand"
    fi
 
    if [ $(mount | grep -c ${_root_device}) -ne 0 ]; then
        detach_from_device ${_root_device}
        success_or_die "Can not detach from ${_root_device}"

        umount ${_root_device}
        success_or_die "Can not unmount root device. Please try to do this by hand"
    fi

    exit $1
}

# Checking previous command result code and undo all
# changes it case of nonzero result
function success_or_clean {
    [ $? -eq 0 ] || {
         echo "$1"
         clean_and_exit 1
    }
}

# Mounting rootfs to temporary dir.
function mount_rootfs_tree {
    [ ! -e "$_rootfs_path" ] && {
    	mkdir $_rootfs_path
        success_or_die "Cannot create the directory to mount rootfs."  
    }
   
    attach_device ${_root_device}                       # Attaching root device to blksnap
    success_or_clean "Can not attach the device ${_root_device}"
    
    if mount | grep -q ${_root_device}; then                        # Check boot and root device. If it is still mounted - 
        umount ${_root_device}
        success_or_die "Can not unmount $_root_device"
    fi

    mount ${_root_device} $_rootfs_path                          # Mounting root device
    success_or_clean "Can not mount root device. Make sure you performed restore correctly."
 
    initialize_stores ${_root_device} ${_rootfs_path}                 # Mapping store and bitmap
    success_or_clean "Can not initialize stores for root device. Make sure you performed restore correctly."
    
    [ ${_boot_device} != "none" ] && {                  # In case if boot device was specified
        [ -e "$_rootfs_path/boot" ] || {                         # do the same steps for boot device
            echo "There is no /boot directory. Make sure you performed restore correctly." 
            clean_and_exit 0
        }

        attach_device ${_boot_device}    
        success_or_clean "Can not attach the device ${_boot_device}"
 
        if mount | grep -q ${_boot_device}; then                        # Check boot and root device. If it is still mounted - 
            umount ${_boot_device}
            success_or_die "Can not unmount $_boot_device"
        fi

        mount ${_boot_device} $_rootfs_path/boot
        success_or_clean "Can not mount boot device. Make sure you performed restore correctly."
    
    	initialize_stores ${_boot_device} ${_rootfs_path}/boot
    	success_or_clean "Can not initialize stores for boot device. Make sure you performed restore correctly."
    }   
}

# This function parsing commnad line.
function parse_command_line { 
    while getopts "hir:b:" opt; do
        case $opt in
            h)
                help; exit 0
                ;;
            r)
                _root_device=$OPTARG
                ;;
            b)
                _boot_device=$OPTARG
                ;;
            i)
                _interactive_mode="yes"
                ;;
            \?)
                help; exit 1
                ;;
        esac
    done
    [ ${_root_device} == "none" ] && {                      # Root device is mandatory
        echo "Root device was not set"
        help
        exit 1
    }
}

# Returns uuid for device path using blkid command
# input param: device path
function get_uuid_from_devpath {
    result=`blkid $1 | cut -d \" -f 2`                      # Set the UUID name for that device

    [ ${#result} -eq 0 ] && {                               # Check the result. 
        echo "UUID for the device $1 was not found. Maybe it does not exists?"
        clean_and_exit 1
    }
    
    echo $result;
}

# Return mount point from device line
# input param:   
function get_mount_point {
    echo $1 | cut -d , -f 2 | tr -d ' '
}

function get_devname {
    echo $1 | cut -d , -f 1 | tr -d ' '
}

# Return values
# 0 - No
# 1 - Yes
function need_modify_mtab {
	release=$(cat $_rootfs_path/etc/*-release)
	
	if [[ "$release" =~ "CentOS" ]] || [[ "$release" =~ "Red Hat" ]]; then
		echo 1
	else 
		echo 0
	fi
}

# This function is returning devname
# for mountpoint in mtab
# input param: mount point
function devname_for_mntdir_mtab {
    echo `cat $_rootfs_path/etc/mtab | grep " $1 " | awk '{print $1}'`
}

# Function changing data on fstab
# and also in mtab if needed
# Input params: 
# 1. Device path in fstab
# 2. UUID of the device
# 3. Device mount point
function change_on_fstab {
    sed -i "s|^${1}|${2}|" -i $_rootfs_path/etc/fstab               # Changes the entry in fstab to UUID form
    mod=$(need_modify_mtab) 
    
    [ $mod == 1 ] && {
        dev_name=$(devname_for_mntdir_mtab $3)
        [ ${#dev_name} -eq 0 ] || sed -i "s|^${dev_name}|${2}|" -i $_rootfs_path/etc/mtab               # Changes the entry in fstab to UUID form
    }
}

function modify_fstab {
    cp $_rootfs_path/etc/fstab $_rootfs_path/etc/fstab.backup
    cat $_rootfs_path/etc/fstab | egrep -v "^#|proc|/dev/fd|/dev/cdrom|devpts|tmpfs|sysfs" | awk '{print $1,",",$2;}' > /tmp/devices   # Stores all /dev entries from fstab into a file
    
    USER_RESPONSE="none"                                                    # Setting user response flag to modify fstab entries
    
    while read LINE; do                                                     # For each line in /tmp/devices
	    mount_point=$(get_mount_point "$LINE")
        if [ x$mount_point == x"/" ] ; then 
            change_on_fstab $(get_devname "$LINE") $_root_device $mount_point
        elif [ x$mount_point == x"/boot" ] ; then

            [ $_boot_device == "none" ] && {
                echo "Error. We have a boot partition but it was not set"
            }
 
            change_on_fstab $(get_devname "$LINE") $_boot_device $mount_point
        elif [ x$mount_point != x"" ] ; then				   #exclude empty strings
            if [ $_interactive_mode == "yes" ] && [ x$USER_RESPONSE == x"none" ] ; then
                printf "You have more them root and boot partitions in fstab. Would you like to enter device path for it? (y/n)" 
                read RESPONSE < /dev/tty                                                        #ask user for respose
                case "$RESPONSE" in
                    [yY]|[yY][eE][sS])                                             
                        echo "b;ah" 
                        USER_RESPONSE="Y" 
                        ;;
                    *)
                        USER_RESPONSE="N"              
                esac
            fi

            if [ $_interactive_mode == "yes" ] && [ x$USER_RESPONSE == x"Y" ] ; then  # Ask user for the device path 
                echo "Enter device path for mountpoint $mount_point"
                read dev_path < /dev/tty
    
                change_on_fstab $(get_devname "$LINE") $dev_path $mount_point
            else                                # Comment lines in fstab if we are not in interactive mode 
                sed "s|^\($(get_devname "$LINE")\)|#\1|" -i $_rootfs_path/etc/fstab  # or customer does not want to modify them

                mod=$(need_modify_mtab)
                [ $mod == 1 ] && {
                    dev_name=$(devname_for_mntdir_mtab $mount_point)
                    [ ${#dev_name} -eq 0 ] || sed "s|^\($dev_name\)|#\1|" -i $_rootfs_path/etc/mtab               # Changes the entry in fstab to UUID form
                }
           fi

        fi

    done </tmp/devices
}

function patch_grub_config {
    [ -e $1 ] || {                                      # File name depends from grub version. It's ok if we did not found it.
        return
    } 
    
    # Replace root entry in grub config
    sed "s/root=[^ ]\+/root=UUID=$(get_uuid_from_devpath $_root_device)/" -i $1
    
    # Delete rd_LVM_LV in grub config
    sed "s/rd_LVM_LV=[^ ]\+/ /g" -i $1
}

function fix_grub {
    patch_grub_config "$_rootfs_path/etc/grub.conf"
    patch_grub_config "$_rootfs_path/boot/grub/grub.conf"
    patch_grub_config "$_rootfs_path/boot/grub/menu.lst"   

    # Do chroot
    mount --bind /proc $_rootfs_path/proc
    mount --bind /dev $_rootfs_path/dev

    cp /opt/apprecovery/scripts/grub_chroot.sh $_rootfs_path/
    chroot $_rootfs_path /grub_chroot.sh

    umount $_rootfs_path/proc
    umount $_rootfs_path/dev
    
    rm $_rootfs_path/grub_chroot.sh
}

[ "$1" ] || help

parse_command_line $@

mount_rootfs_tree

modify_fstab

fix_grub

clean_and_exit 0
